import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { AppDispatch, RootState } from './store';
import { GraphQueryCountResultFromBackend, GraphQueryResultMetaFromBackend, NodeAttributes, QueryStatusResult } from 'ts-common';
export type UniqueEdge = {
  attributes: NodeAttributes;
  source: string;
  target: string;
  count: number;
};

export type GraphQueryResultStatus = 'idle' | 'error' | 'querying' | 'success' | 'aborted' | 'countAborted';
// Define a type for the slice state
export type GraphQueryResult = {
  graph: GraphQueryResultMetaFromBackend;
  graphCounts?: GraphQueryCountResultFromBackend;
  status: GraphQueryResultStatus;
  currentQueryCallID: string;
};

// Define the initial state using that type
export const initialState: GraphQueryResult = {
  graph: {
    metaData: {
      nodes: { count: 0, labels: [], types: {} },
      edges: { count: 0, labels: [], types: {} },
      topological: { density: 0, self_loops: 0 },
    },
    nodes: [],
    edges: [],
  },
  graphCounts: undefined,
  status: 'idle',
  currentQueryCallID: '',
};

export function parseValue(value: number | string | Array<any> | object) {
  switch (value.constructor) {
    case Number:
      return (value as number).toFixed(2);
    case String:
      return value as string;
    case Array:
      return (value as Array<any>).map(String).join(', ');
    case Object:
      return JSON.stringify(value as object);
  }

  return null;
}

export const graphQueryResultSlice = createSlice({
  name: 'graphQueryResult',
  initialState,
  reducers: {
    setNewGraphQueryResult: (
      state,
      action: PayloadAction<{ data: QueryStatusResult<GraphQueryResultMetaFromBackend>; callID: string }>,
    ) => {
      if (state.status !== 'querying') {
        console.log('Set new graph query result called when not querying', state.status);
        return;
      }
      if (state.currentQueryCallID !== action.payload.callID) {
        console.log('Set new graph query result called with wrong callID', state.currentQueryCallID, action.payload.callID);
        return;
      }

      state.graph = action.payload.data.result.payload;
      state.status = 'success';
      state.graphCounts = undefined;
    },
    setNewGraphQueryCountResult: (
      state,
      action: PayloadAction<{ data: QueryStatusResult<GraphQueryCountResultFromBackend>; callID: string }>,
    ) => {
      if (state.status !== 'success') {
        console.log("Set new count graph query result called when the normal result set isn't there yet", state.status);
        return;
      }
      if (state.currentQueryCallID !== action.payload.callID) {
        console.log('Set new count graph query result called with wrong callID', state.currentQueryCallID, action.payload.callID);
        return;
      }

      state.graphCounts = action.payload.data.result.payload;
    },
    resetGraphQueryResults: (state, action: PayloadAction<undefined | { status: GraphQueryResultStatus }>) => {
      state.graph = initialState.graph;
      state.graphCounts = undefined;
      state.status = action.payload?.status ?? 'success';
    },
    queryingBackend: (state, action: PayloadAction<{ status: GraphQueryResultStatus; callID: string }>) => {
      state.status = action.payload.status ?? 'querying';
      state.currentQueryCallID = action.payload.callID;
    },
    abortedBackend: (state, action: PayloadAction<{ type: 'result' | 'count'; callID: string }>) => {
      if (state.status !== 'querying' || state.currentQueryCallID !== action.payload.callID) {
        console.debug('Aborted backend call does not match current call', state.status, state.currentQueryCallID, action.payload.callID);
        return;
      }
      if (action.payload.type === 'result') {
        state.graph = initialState.graph;
        state.graphCounts = undefined;
        state.status = 'aborted';
      }
      if (action.payload.type === 'count') {
        state.status = 'countAborted';
        state.graphCounts = undefined;
      }
    },
  },
});

export const { resetGraphQueryResults, queryingBackend, setNewGraphQueryCountResult, abortedBackend } = graphQueryResultSlice.actions;

/**
 * Asynchronously sets a new graph query result.
 * Prefer to use this then the synchronous version since if the result is very large,
 *  the entire website will freeze while graphQueryBackend2graphQuery runs.
 *
 * This function dispatches an action to update the graph query result
 * in the store with the provided payload.
 *
 * @param payload - The result of the graph query to be set.
 * @param dispatcher - The dispatch function to trigger the action.
 */
export const asyncSetNewGraphQueryResult = async (
  payload: { data: QueryStatusResult<GraphQueryResultMetaFromBackend>; callID: string },
  dispatcher: AppDispatch,
) => {
  dispatcher(graphQueryResultSlice.actions.setNewGraphQueryResult(payload));
};

// Other code such as selectors can use the imported `RootState` type
export const selectGraphQuery = (state: RootState) => state.graphQueryResult;
export const selectGraphQueryResult = (state: RootState) => state.graphQueryResult.graph;
export const selectGraphQueryResultNodes = (state: RootState) => state.graphQueryResult.graph.nodes;
export const selectGraphQueryResultLinks = (state: RootState) => state.graphQueryResult.graph.edges;
export const selectGraphQueryResultMetaData = (state: RootState) => state.graphQueryResult.graph.metaData;
export const selectGraphQueryCounts = (state: RootState) => state.graphQueryResult.graphCounts;

export default graphQueryResultSlice.reducer;
